/*
 * Copyright 2013 Takashi Ogura <tarchan at mac.com>.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.mac.tarchan.twoface;

import java.net.URL;
import java.util.ResourceBundle;
import javafx.beans.InvalidationListener;
import javafx.beans.Observable;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Button;
import javafx.scene.control.TextArea;
import javafx.scene.image.ImageView;
import javafx.scene.layout.AnchorPane;
import javafx.stage.Stage;

/**
 * FXML Controller class
 *
 * @author Takashi Ogura <tarchan at mac.com>
 */
public class ErrorDialogController implements Initializable {

    @FXML
    private AnchorPane error;
    @FXML
    private TextArea message;
    @FXML
    private ImageView icon;
    @FXML
    private Button close;
    private StringProperty title = new SimpleStringProperty(this, "title");

    /**
     * Initializes the controller class.
     */
    @Override
    public void initialize(URL url, ResourceBundle rb) {
        error.sceneProperty().addListener(new InvalidationListener() {
            @Override
            public void invalidated(Observable o) {
                Stage stage = (Stage) error.getScene().getWindow();
                if (stage != null) {
                    stage.titleProperty().bind(title);
                }
            }
        });
    }

    public StringProperty titleProperty() {
        return title;
    }

    public StringProperty messageProperty() {
        return message.textProperty();
    }

    @FXML
    private void handleClose(ActionEvent event) {
        Stage stage = (Stage) error.getScene().getWindow();
        stage.close();
    }
}
